rm(list = ls())
setwd('/Users/mahradvaghefi/Desktop/empirical_analysis_2_11_2019/forecasting')
source('hdknockoff_all_1.R')
require(methods)
library(readxl)
library(glmnet)
library(MASS) 
library(Matrix)
library(knockoff)
library(rARPACK)
library(doParallel)
data <- read_excel('StockWatson.xlsx',sheet = 1,col_types = 'numeric')
data <- data[6:nrow(data),]
data2 <- read_excel('StockWatson.xlsx',sheet = 2)
data3 <- read_excel('StockWatson.xlsx',sheet = 3)
data3 <- as.matrix(data3)
Time = seq(ISOdate(1960,5,1), ISOdate(2008,11,1), by = "quarter")
Time = trunc(Time, units = "days")
Time = as.POSIXct(Time)
Y = cbind(data$GDP251,data$CPIAUCSL,data$GDP264,data$IPS13,data$CES048,data$LHU680,data$HSSOU,
          data$PMNO,data$GDP275_3,data$LBMNU,data$FYFF,data$CCINRV,data$EXRCAN,data$FSDJ,data$HHSNTN)
colnames(Y) = c('GDP251','CPIAUCSL','GDP264','IPS13','CES048','LHU680','HSSOU',
                'PMNO','GDP275_3','LBMNU','FYFF','CCINRV','EXRCAN','FSDJ','HHSNTN')
#data = cbind(Time,data)
Z = as.matrix(data[,as.logical(data2$E)])
W = 120 # length of rolling window
it = 100 # number of iteration for IPAD1 and IPAD2
SelTrack = matrix(0,nrow(data)-W-1,ncol(Z)) # A matrix for tracking selected variables at which point in time
colnames(SelTrack) = colnames(Z)
Track1 = list(GDP251 = SelTrack, CPIAUCSL = SelTrack, GDP264 = SelTrack, IPS13 = SelTrack,
              CES048 = SelTrack, LHU680 = SelTrack, HSSOU = SelTrack, PMNO = SelTrack, 
              GDP275_3 = SelTrack, LBMNU = SelTrack, FYFF = SelTrack, CCINRV = SelTrack,
              EXRCAN = SelTrack, FSDJ = SelTrack, HHSNTN = SelTrack)

Model = c('AR1','FAR1','Lasso','IPAD')

yhat = matrix(NA,nrow(data)-W-1,ncol(Y))
for(j in 1:ncol(Y)){
  y = as.matrix(Y[,j])
  X = y
  for(i in 1:(nrow(data)-W-1)){
    sX = as.matrix(X[i:(W+i-1),])
    sy = y[(i+1):(W+i),]
    ones <- matrix(1,nrow(sX) , 1)
    sZ <- cbind(ones,sX)
    Beta <- solve(t(sZ) %*% sZ, t(sZ) %*% sy)
    yhat[i,j] <- c(1,X[(W+i),]) %*% Beta
  }
}
AR_yhat = yhat

yhat = matrix(NA,nrow(data)-W-1,ncol(Y))
rmax = 10 # maximum number of factors
PC = matrix(NA,rmax,1)
for(j in 1:ncol(Y)){
  y = as.matrix(Y[,j])
  index = which(colnames(Z)==colnames(Y)[j])
  if (length(index) != 0){
    SZ = Z[,-index]
  }else {
    SZ = Z
  }
  SZ = scale(SZ)
  N <- ncol(SZ)
  TT <- nrow(SZ)
  mZZ = SZ %*%t(SZ)
  for(k in rmax:1){
    eig <- eigs_sym(mZZ, k, which = "LM", sigma = NULL, lower = TRUE)
    meigvec <- eig$vectors 
    mF <- sqrt(TT) * meigvec       # estimated factors
    Lam <- (t(mF) %*% SZ)/TT
    if(k==rmax){
      sigma2 = sum((SZ-mF %*% Lam)^2)/(N*TT)
    }
    PC[k,] = sum((SZ-mF %*% Lam)^2)/(N*TT) + k*sigma2*((N+TT)/(N*TT))*log((N*TT)/(N+TT))
  }
  r = which(PC == min(PC))
  eig <- eigs_sym(mZZ, r, which = "LM", sigma = NULL, lower = TRUE)
  meigvec <- eig$vectors 
  mF <- sqrt(TT) * meigvec
  X = cbind(y,mF)
  for(i in 1:(nrow(data)-W-1)){
    sX = X[i:(W+i-1),]
    sy = y[(i+1):(W+i),]
    ones <- matrix(1,nrow(sX) , 1)
    sZ <- cbind(ones,sX)
    Beta <- solve(t(sZ) %*% sZ, t(sZ) %*% sy)
    yhat[i,j] <- c(1,X[(W+i),]) %*% Beta
  }
}
FAR_yhat = yhat

cl <- makeCluster(detectCores())
registerDoParallel(cl)
yhat = matrix(NA,nrow(data)-W-1,ncol(Y))
for(j in 1:ncol(Y)){
  y = as.matrix(Y[,j])
  index = which(colnames(Z)==colnames(Y)[j])
  if (length(index) != 0){
    X = cbind(y,mF,Z[,-index])
  }else {
    X = cbind(y,mF,Z)
  }
  mres <- foreach(i = 1:(nrow(data)-W-1),.combine = rbind, .inorder = TRUE,
                  .packages = c("glmnet","Matrix")) %dopar% {
                    sX = X[i:(W+i-1),]
                    sy = y[(i+1):(W+i),]
                    Op_lambda <- cv.glmnet(sX,sy)$lambda.min
                    fit <- glmnet(sX,sy,lambda = Op_lambda)
                    a0 <- fit$a0
                    Beta <- fit$beta
                    a0 <- as.vector(a0)
                    Beta <- as.vector(Beta)
                    Beta <- c(a0,Beta)
                    yf <- c(1,X[(W+i),]) %*% Beta
                  }
  yhat[,j] <- mres
}
Lasso_yhat = yhat


yhat = matrix(NA,nrow(data)-W-1,ncol(Y))
IPAD_yhat = matrix(0,nrow(data)-W-1,ncol(Y))
for(t in 1:it){
  for(j in 1:ncol(Y)){
    y = as.matrix(Y[,j])
    index = which(colnames(Z)==colnames(Y)[j])
    if (length(index) != 0){
      X = Z[,-index]
    }else {
      X = Z
    }
    D = y
    mres <- foreach(i = 1:(nrow(data)-W-1),.combine = rbind, .inorder = TRUE,
                    .packages = c("glmnet","Matrix","rARPACK")) %dopar% {
                      sX = X[i:(W+i-1),]
                      sD = D[i:(W+i-1),]
                      M = diag(1,W,W) - sD %*% solve(t(sD)%*%sD)%*%t(sD)
                      sy = y[(i+1):(W+i),]
                      sye = M %*% sy
                      sXe = M %*% sX
                      Selset = hdknockoff(X = sXe,y = sye,q = 0.2, knockx_set = "facrand")
                      Supp = Selset$S
                      ones <- matrix(1,nrow(sX),1)
                      sZ <- cbind(ones,sD,sX[,Supp])
                      Beta <- solve(t(sZ) %*% sZ, t(sZ) %*% sy)
                      yf <- c(1,D[(W+i),],X[(W+i),Supp]) %*% Beta
                      var_sel <- rep(0,ncol(Z))
                      var_sel[Supp] <- 1
                      c(yf,var_sel)
                    }
    yhat[,j] <- mres[,1]
    Track1[[j]] <- mres[,-1]+Track1[[j]]
  }
  IPAD_yhat = yhat+IPAD_yhat
}
IPAD_yhat = IPAD_yhat/it
stopCluster(cl)

ef1 = AR_yhat - Y[(W+2):nrow(data),]
ef2 = FAR_yhat - Y[(W+2):nrow(data),]
ef3 = Lasso_yhat - Y[(W+2):nrow(data),]
ef4 = IPAD_yhat - Y[(W+2):nrow(data),]
d_f1 = ef4^2 - ef1^2 
d_f2 = ef4^2 - ef2^2 
d_f3 = ef4^2 - ef3^2
DM = matrix(NA,3,ncol(Y))
colnames(DM) = c('GDP251','CPIAUCSL','GDP264','IPS13','CES048','LHU680','HSSOU',
                 'PMNO','GDP275_3','LBMNU','FYFF','CCINRV','EXRCAN','FSDJ','HHSNTN')
row.names(DM) = c('IPADvsAR','IPADvsFAR','IPADvsLasso')
for(j in 1:ncol(Y)){
  DM[1,j] = sqrt(nrow(AR_yhat))*mean(d_f1[,j])/sqrt(var(d_f1[,j]))
  DM[2,j] = sqrt(nrow(FAR_yhat))*mean(d_f2[,j])/sqrt(var(d_f2[,j]))
  DM[3,j] = sqrt(nrow(Lasso_yhat))*mean(d_f3[,j])/sqrt(var(d_f3[,j]))
}
RMSE = matrix(NA,4,ncol(Y))
colnames(RMSE) = c('GDP251','CPIAUCSL','GDP264','IPS13','CES048','LHU680','HSSOU',
                   'PMNO','GDP275_3','LBMNU','FYFF','CCINRV','EXRCAN','FSDJ','HHSNTN')
row.names(RMSE) = c('AR','FAR','Lasso','IPAD1')
for(j in 1:ncol(Y)){
  RMSE[1,j] = sqrt(mean(ef1[,j]^2))
  RMSE[2,j] = sqrt(mean(ef2[,j]^2))
  RMSE[3,j] = sqrt(mean(ef3[,j]^2))
  RMSE[4,j] = sqrt(mean(ef4[,j]^2))
}
save.image(file = 'Result.RData')