hdknockoff <- function(X,y,q = 0.2,lambda = "CV", knockx_set = "facrand"){
  
  if (knockx_set == "facrand"){
    n = nrow(X)
    p = ncol(X)
    rmax = 10 # maximum number of factors
    #ER = matrix(NA,rmax,1)
    #mXX = X0 %*% t(X0)/(n0*P)
    #eig <- eigs_sym(mXX, rmax+1, which = "LM", sigma = NULL, lower = TRUE)
    #for(k in 1:rmax){
    #  ER[k,] = eig$values[k]/eig$values[k+1]
    #}
    #r = which(ER == max(ER))
    PC = matrix(NA,rmax+1,1)
    SX = scale(X)
    mXX = SX %*%t(SX)
    for(k in rmax:0){
      if (k == 0){
        PC[k+1,] = sum(SX^2/(n*p))
      } else 
      eig <- eigs_sym(mXX, k, which = "LM", sigma = NULL, lower = TRUE)
      meigvec <- eig$vectors 
      mF <- sqrt(n) * meigvec       # estimated factors
      Lam <- (t(mF) %*% SX)/n
      if(k==rmax){
        sigma2 = sum((SX-mF %*% Lam)^2)/(n*p)
      }
      PC[k+1,] = sum((SX-mF %*% Lam)^2)/(n*p) + k*sigma2*((n+p)/(n*p))*log((n*p)/(n+p))
    }
    r = which(PC == min(PC))-1
    # IC = matrix(NA,rmax+1,1)
    # SX = scale(X)
    # mXX = SX %*%t(SX)
    # for(k in rmax:0){
    #   if (k == 0){
    #     IC[k+1,] = log(sum(SX^2/(n*p))) + k*((n+p)/(n*p))*log((n*p)/(n+p))
    #   } else {
    #   eig <- eigs_sym(mXX, k, which = "LM", sigma = NULL, lower = TRUE)
    #   meigvec <- eig$vectors 
    #   mF <- sqrt(n) * meigvec       # estimated factors
    #   Lam <- (t(mF) %*% SX)/n
    #   IC[k+1,] = log(sum((SX-mF %*% Lam)^2)/(n*p)) + k*((n+p)/(n*p))*log((n*p)/(n+p))
    #   }
    # }
    # r = which(IC == min(IC))-1
    if (r != 0){
      C = fFact(X,r)
      res = X - C
      sd = sqrt(sum(res^2)/(n*p))
      nres = matrix(rnorm(n*p,sd = sd),n,p)
      X_ko = C + nres
    }else{
      res = X
      sd = sqrt(sum(res^2)/(n*p))
      nres = matrix(rnorm(n*p,sd = sd),n,p)
      X_ko = nres
    }
    Xnew = cbind(X, X_ko)
    p_sel = ncol(Xnew)/2
    if (lambda == "CV") {
      # Finding Optimal value of Lambda
      Op_lambda <- cv.glmnet(Xnew,y)$lambda.min
      # apply glmnet to find beta vector under original scale
      Beta <- glmnet(Xnew,y,lambda = Op_lambda)$beta
      Beta <- as.vector(Beta)
      # beta vector under aligned scale
      stan_X = scale(Xnew, center=F, scale=T)/sqrt(n-1)
      y.norm = attr(stan_X,"scale")*sqrt(n-1)
      betap.stan = Beta*y.norm
    }
    
    # Calculate knockoff test statistics W_j
    W = abs(betap.stan[1:p_sel]) - abs(betap.stan[(p_sel+1):(2*p_sel)])

    t = sort(c(0, abs(W)))
    ratio = c(rep(0, p))
    
    # find the knockoff threshold T1
    for (j in 1:p) {
      ratio[j] = sum(W <= -t[j])/max(1, sum(W >= t[j]))
    }
    id = which(ratio <= q)[1]
    if(length(id) == 0){
      T1 = Inf
    } else {
      T1 = t[id]
    }
    
    # set of discovered variables by knockoff threshold T_1
    S = which(W >= T1)
    
    
    # find the knockoff_plus threshold T2
    for (j in 1:p) {
      ratio[j] = (1+sum(W <= -t[j]))/max(1, sum(W >= t[j]))
    }
    id = which(ratio <= q)[1]
    if(length(id) == 0){
      T2 = Inf
    } else {
      T2 = t[id]
    }
    
    # set of discovered variables
    S_plus = which(W >= T2)
    obj = obj = list(S = S, S_plus = S_plus, r = r)
  }
  return(obj)
}

##########################################################

fFact <- function(X, r){
  obj <- svds(X,r) #nu = r,nv = r)
  C <- obj$u %*% diag(obj$d,length(obj$d),length(obj$d)) %*% t(obj$v)  
  return(C)
}

##########################################################
## splite the sample into two disjoint group
sample.Splite = function(X, y){
  n   = nrow(X)
  p   = ncol(X)
  n0  = n/2
  n1  = n-n0
  full.sample   = c(1:n)
  scr.sample    = sample(full.sample,n0)
  sel.sample    = setdiff(full.sample,scr.sample)
  X_0           = X[scr.sample,]
  y_0           = y[scr.sample]
  X_1           = X[sel.sample,]
  y_1           = y[sel.sample]
  obj  = list(X0 = X_0, y0 = y_0, X1 = X_1, y1 = y_1)
  return(obj)
}

###########################################################################
# Part II: additional functions
# Part II 1) fdr: calculating fdr

fdr <- function(S, beta.true) {
  fdp = sum(beta.true[S] == 0)/max(1, length(S))
  return(fdp)
}
###########################################################################

###########################################################################
# Part II: additional functions
# Part II 2) pow: calculating power

pow <- function(S, beta.true) {
  tdp = sum(beta.true[S] != 0)/max(1,sum(beta.true != 0))
  return(tdp)
}

###########################################################################
