# Benchmark Model
rm(list = ls())
ptm <- proc.time() # Start the clock

setwd('/Users/mahradvaghefi/Desktop/empirical_analysis_2_11_2019/simulation')
source('hdknockoff_all_Factor.R')

# Loading packages --------------------------------------------------------
library(rARPACK)
library(doParallel)
require(glmnet)
require(graphics)
library(MASS) 
library(Matrix)
library(knockoff)

# set the number of cores for Parallel computing --------------------------

#registerDoParallel(cores=2)

# Initial variables -------------------------------------------------------
load("Data.RData")
data <- data[6:nrow(data),]
X = as.matrix(data[,as.logical(data2$E)])
knockx_set <- c("facrand","BCknockoff1","hdBCknockoff1") # set of methods for generating knockoffX
r_set <- 3 # set for number of comment factors
c_set <- c(0.2,0.3,0.5) # set for signal to noise ratio of dependent variable y 
N_set <- nrow(X) # Set for number of observations
P_set <- ncol(X) # Set for number of parameters
S_set <- 10 # Set for number of true signals
A_set <- 4 # Set for Signal Amplitude
q_set <- 0.2 # Set for Target FDR level
it <- 100 # number of iterations

Re = array(list(),dim = c(length(N_set),length(P_set),length(r_set),
                          length(c_set),length(S_set),length(A_set),
                          length(q_set),length(knockx_set)))
MeanRe = array(list(),dim = c(length(N_set),length(P_set),length(r_set),
                              length(c_set),length(S_set),length(A_set),
                              length(q_set),length(knockx_set)))
# Constructing Simulated Data ---------------------------------------------
for (i1 in 1:length(N_set)){
  N <- N_set[i1]
  for (i2 in 1:length(P_set)){
    P <- P_set[i2]
    for (i3 in 1:length(r_set)){
      r <- r_set[i3]
      for (i4 in 1:length(c_set)){
        c <- c_set[i4]
        for (i5 in 1:length(S_set)){
          S <- S_set[i5]
          for (i6 in 1:length(A_set)){
            A <- A_set[i6]
            for (i7 in 1:length(q_set)){
              q <- q_set[i7]
              for (i8 in 1:length(knockx_set)){
                knockx <- knockx_set[i8]
                cl <- makeCluster(detectCores()) 
                registerDoParallel(cl)
                mres <- foreach(i = 1:it, .combine = cbind, .inorder = TRUE, 
                                .packages = c("glmnet","knockoff","MASS","rARPACK")) %dopar% {
                                  X <- scale(X,center = FALSE)/sqrt(N-1)
                                  
                                  Gy <- DGPy(X, s = S, A = A, c = c)
                                  y <- Gy$y
                                  sig2no_1 <- Gy$sig_noise
                                  B.true <- Gy$Beta
                                  if (knockx == "hdBCknockoff1" || knockx == "hdBCknockoff2"){
                                    Obj <- hdknockoffX(X = X,y=y,beta.true = B.true,method = knockx)
                                    fdp_1 <- Obj$fdp
                                    fdp_2 <- Obj$fdp_plus
                                    tdp_1 <- Obj$tdp
                                    tdp_2 <- Obj$tdp_plus
                                  } else {
                                    Xnew <- hdknockoffX(X,y,r,method = knockx)
                                    B.true <- Gy$Beta
                                    # run high-dimensional knockoff (HDKnockoff) ------------------------------
                                    
                                    Supp = hdknockoff(Xnew, y, q,lambda = "CV")
                                    fdp_1 = fdr(Supp, B.true)
                                    tdp_1 = pow(Supp, B.true)
                                    
                                    # run high-dimensional knockoff+ (HDKnockoff+) ----------------------------
                                    
                                    Supp_plus = hdknockoff_plus(Xnew, y, q,lambda = "CV")
                                    fdp_2 = fdr(Supp_plus, B.true)
                                    tdp_2 = pow(Supp_plus, B.true)
                                  }
                                  
                                  # stack variables --------------------------------------------------------
                                  
                                  c(fdp_1, tdp_1, fdp_2, tdp_2,sig2no_1)#,fdp_3, tdp_3, fdp_4, tdp_4)
                                }
                
                stopCluster(cl)
                fdp <- as.matrix(mres[1,])
                tdp <- as.matrix(mres[2,])
                fdp_plus <- as.matrix(mres[3,])
                tdp_plus <- as.matrix(mres[4,])
                sig2no <- as.matrix(mres[5,])
            
                # Final Results -----------------------------------------------------------
                
                mean_fdp <- apply(fdp, 2, mean)
                mean_tdp <- apply(tdp, 2, mean)
                mean_fdp_plus <- apply(fdp_plus, 2, mean)
                mean_tdp_plus <- apply(tdp_plus, 2, mean)
                mean_sig2no <- apply(sig2no, 2, mean)
                result <- cbind(fdp,tdp,fdp_plus,tdp_plus,sig2no)
                mean_result <- rbind(mean_fdp,mean_tdp,mean_fdp_plus,mean_tdp_plus,mean_sig2no)
                MeanRe[i1,i2,i3,i4,i5,i6,i7,i8] = list(Mean_Result = mean_result)
              }
            }
          }
        }
      }
    }
  }
}
save(Re, MeanRe,file = 'Result.RData')
proc.time() - ptm # stop the clock