# Benchmark Model
rm(list = ls())
ptm <- proc.time() # Start the clock

setwd('/Users/mahradvaghefi/Desktop/simulation_section_02_11_2019/nonlinear_design')
source('hdknockoff_all_Forest.R')

# Loading packages --------------------------------------------------------
library(randomForest)
library(PerMallows)
library(rARPACK)
library(doParallel)
require(glmnet)
require(graphics)
library(MASS) 
library(Matrix)
library(knockoff)

# set the number of cores for Parallel computing --------------------------

#registerDoParallel(cores=2)

# Initial variables -------------------------------------------------------

dgx_set <- c("fac1","fac2","normal") # set of methods for generating design matrix X
knockx_set <- "facrand" # set of methods for generating knockoffX
r_set <- 3 # set for number of comment factors
c_set <- c(0.1,0.2,0.3) # set for signal to noise ratio of dependent variable y 
theta_set <- c(1,2) # set for signal to noise ratio of design matrix X
N_set <- 1000 # Set for number of observations
P_set <- c(50,250,500) # Set for number of parameters
S_set <- 10 # Set for number of true signals
A_set <- 4 # Set for Signal Amplitude
Ru_set <- 0 # Set for Degree of dependence between covariates
q_set <- 0.2 # Set for Target FDR level
it <- 100 # number of iterations

Re = array(list(),dim = c(length(N_set),length(P_set),length(r_set),
                          length(theta_set),length(c_set),length(Ru_set),
                          length(S_set),length(A_set),length(q_set),
                          length(dgx_set),length(knockx_set)))
MeanRe = array(list(),dim = c(length(N_set),length(P_set),length(r_set),
                              length(theta_set),length(c_set),length(Ru_set),
                              length(S_set),length(A_set),length(q_set),
                              length(dgx_set),length(knockx_set)))
# Constructing Simulated Data ---------------------------------------------
for (i1 in 1:length(N_set)){
  N <- N_set[i1]
  for (i2 in 1:length(P_set)){
    P <- P_set[i2]
    for (i3 in 1:length(r_set)){
      r <- r_set[i3]
      for (i4 in 1:length(theta_set)){
        theta <- r* theta_set[i4]
        for (i5 in 1:length(c_set)){
          c <- c_set[i5]
          for (i6 in 1:length(Ru_set)){
            Ru <- Ru_set[i6]
            for (i7 in 1:length(S_set)){
              S <- S_set[i7]
              for (i8 in 1:length(A_set)){
                A <- A_set[i8]
                for (i9 in 1:length(q_set)){
                  q <- q_set[i9]
                  for (i10 in 1:length(dgx_set)){
                    dgx <- dgx_set[i10]
                    for (i11 in 1:length(knockx_set)){
                      knockx <- knockx_set[i11]
                      cl <- makeCluster(detectCores()) 
                      registerDoParallel(cl)
                      mres <- foreach(i = 1:it, .combine = cbind, .inorder = TRUE, 
                                      .packages = c("glmnet","knockoff","MASS","rARPACK","PerMallows","randomForest")) %dopar% {
                                        GX <- DGPX(n = N, p = P, r = r, ru = Ru, theta = theta, method = dgx)
                                        X <- GX$X
                                        X <- scale(X,center = FALSE)/sqrt(N-1)

                                        Gy <- DGPy(X, s = S, A = A, c = c, method = "nonlinear2")
                                        y <- Gy$y
                                        Xnew <- hdknockoffX(X,y,r,method = knockx)
                                        B.true <- Gy$Beta
                                        # run high-dimensional knockoff (HDKnockoff) ------------------------------
                                        
                                        Supp = forestknockoff(Xnew, y, q)
                                        fdp_1 = fdr(Supp, B.true)
                                        tdp_1 = pow(Supp, B.true)
                                        
                                        # run high-dimensional knockoff+ (HDKnockoff+) ----------------------------
                                        
                                        Supp_plus = forestknockoff_plus(Xnew, y, q)
                                        fdp_2 = fdr(Supp_plus, B.true)
                                        tdp_2 = pow(Supp_plus, B.true)

                                        # stack variables --------------------------------------------------------
                                        
                                        c(fdp_1, tdp_1, fdp_2, tdp_2)
                                      }
                      
                      stopCluster(cl)
                      fdp <- as.matrix(mres[1,])
                      tdp <- as.matrix(mres[2,])
                      fdp_plus <- as.matrix(mres[3,])
                      tdp_plus <- as.matrix(mres[4,])

                      # Final Results -----------------------------------------------------------
                      
                      mean_fdp <- apply(fdp, 2, mean)
                      mean_tdp <- apply(tdp, 2, mean)
                      mean_fdp_plus <- apply(fdp_plus, 2, mean)
                      mean_tdp_plus <- apply(tdp_plus, 2, mean)
                      result <- cbind(fdp,tdp,fdp_plus,tdp_plus)
                      mean_result <- rbind(mean_fdp,mean_tdp,mean_fdp_plus,mean_tdp_plus)
                      Re[i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11] = list(Result = result)
                      MeanRe[i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11] = list(Mean_Result = mean_result)
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
}
save(Re, MeanRe,file = 'Result.RData')
proc.time() - ptm # stop the clock
